/***************************************************************************
                          system.h  -  hydraulic network model
                             -------------------
    begin                : Sun Apr 9 2000
    copyright            : (C) 2000 by sdteffen
    email                : macke@heini.fbbwu.fh-lueneburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GHYDRAULIC_SYSTEM_H
#define GHYDRAULIC_SYSTEM_H

#include "node.h"
#include "line.h"

//libxml stuff
#include <tree.h>
#include <parser.h>

#include <math.h>

class ghSystem
{
	public:
	//Constructor
	ghSystem(const char *filename);
  //matrix subtraction
  void eliminateGauss();
  /** analyses the Systems and returns the 
			Number of Iterations needed. In case
			of Error, 0 is returned */
  int analyze();
  /** prints Information about the System */
  void print();
  /** Destructor, freeing the allocated arrays */
  ~ghSystem();
	
	private:
	// Variables
	//one-dimensional Array containing the node objects
	ghNode **ghNodes;
	//one-dimensional Array containing the line objects
	ghLine **ghLines;
	/* Actual Number of Nodes in the System*/
	int iNumNodes;
  /* Actual Number of Lines in the System*/
	int iNumLines;
	//TODO: make this accessible through XML
	double fMolarMass; //kg/kmol
	//TODO: make this accessible through XML
	double fNetworkRoughness;
	const static double fAmbientPressure = 101300.0; //Pa
	const static double fTemperature = 10.0; // deg Celsius
	double fRho; // kg/m^3
	//TODO: make this accessible through XML
	const static double fPermissibleNodeError = 0.001;
  // matrix for calculation
  double **fCoeff;
  // Elimination Order for Gauss
  int *iEliminationOrder;
  /* Number of Coefficients */
  int *iNumCoeff;
  /** TODO: make this a local variable */
  double fMaxPressureDifference;
  /** DruckverlustExponent Epsilon */
  double fMinPressureLossExponent;
  /** Druckverlustexponent Epsilon */
  double fMaxPressureLossExponent;
  /** kinematic Viscosity in Pa*s TODO: check if this is equal N*s/m^2
  make this accessible through XML */
  float fKinematicViscosity;
  /** in degree Celsius */
  float fAmbientTemperature;

  //Functions
	void parseNodes(xmlNodePtr xnpXmlNodesNode);
	void parseLines(xmlNodePtr xnpXmlLinesNode);
	/* runs one iteration, returns the maximum node error */
	double runIteration(int iNumIterations);
	//returns the maximum of the two parameters
	double minimum(double value1, double value2);
  //returns the minimum of the two parameters
	double maximum(double value1, double value2);
  /** debug functions that prints the coefficient matrix */
  void printCoeffMatrix();
  /** debug functions dumps the node error array */
  void printNodeErrors(double *fNodeErrors);
};


#endif






